package rainbeau.mithwoodforest.RMFBlocks;

import java.util.List;
import java.util.Random;

import com.google.common.collect.Lists;

import net.minecraft.block.Block;
import net.minecraft.block.BlockLeaves;
import net.minecraft.block.BlockPlanks.EnumType;
import net.minecraft.block.properties.IProperty;
import net.minecraft.block.state.BlockStateContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Blocks;
import net.minecraft.init.Items;
import net.minecraft.item.Item;
import net.minecraft.item.ItemBlock;
import net.minecraft.item.ItemStack;
import net.minecraft.stats.StatList;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraftforge.fml.common.registry.GameRegistry;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

public class BlockMithwoodLeaves extends BlockLeaves {
	
	public BlockMithwoodLeaves(String unlocalizedName, float hardness, float resistance) {
		super();
		this.setUnlocalizedName(unlocalizedName);
	    this.setRegistryName(unlocalizedName);
		this.setDefaultState(blockState.getBaseState().withProperty(DECAYABLE, Boolean.valueOf(true)).withProperty(CHECK_DECAY, Boolean.valueOf(true)));
		this.leavesFancy = true;
		this.setHardness(hardness);
		this.setResistance(resistance);
	    GameRegistry.register(this);
	    GameRegistry.register(new ItemBlock(this).setRegistryName(this.getRegistryName()));
	}

	@Override
	protected BlockStateContainer createBlockState() {
		return new BlockStateContainer(this, new IProperty[] {DECAYABLE, CHECK_DECAY});
	}

	@Override
	public IBlockState getStateFromMeta(int meta) {
		return getDefaultState().withProperty(DECAYABLE, Boolean.valueOf((meta & 4) == 0)).withProperty(CHECK_DECAY, Boolean.valueOf((meta & 8) > 0));
	}

	@Override
	public int getMetaFromState(IBlockState state) {
		int meta = 0;
		if (!state.getValue(DECAYABLE).booleanValue()) {
			meta |= 4;
		}
		if (state.getValue(CHECK_DECAY).booleanValue()) {
			meta |= 8;
		}
		return meta;
	}

	@Override
	public EnumType getWoodType(int meta) {
		return null;
	}

	@SideOnly(Side.CLIENT)
	@Override
	public void setGraphicsLevel(boolean fancy) {
		super.setGraphicsLevel(true);
	}

	@Override
	public Item getItemDropped(IBlockState state, Random rand, int fortune) {
		return Item.getItemFromBlock(BlocksCreate.mithwood_sapling);
	}

	@Override
	public void harvestBlock(World world, EntityPlayer player, BlockPos pos, IBlockState state, TileEntity tile, ItemStack stack) {
		if (!world.isRemote && stack != null && stack.getItem() == Items.SHEARS) {
			player.addStat(StatList.getBlockStats(this));
		}
		else super.harvestBlock(world, player, pos, state, tile, stack);
	}

	@Override
	public boolean isShearable(ItemStack item, IBlockAccess world, BlockPos pos) {
		return item != null;
	}

	@Override
	public List<ItemStack> onSheared(ItemStack item, IBlockAccess world, BlockPos pos, int fortune) {
		return Lists.newArrayList(new ItemStack(this));
	}
	
	@Override
	public void updateTick(World world, BlockPos pos, IBlockState state, Random rand) {
		super.updateTick(world, pos, state, rand);
		dropApple(world, pos);
	}

	public static void dropApple(World world, BlockPos pos) {
		if (world.getBlockState(pos).getBlock() instanceof BlockMithwoodLeaves) {
			BlockMithwoodLeaves leaves = (BlockMithwoodLeaves) world.getBlockState(pos).getBlock();
			Block blockdown = world.getBlockState(pos.down()).getBlock();
			if (blockdown == Blocks.AIR) {
				if (new Random().nextInt(8192) == 0) {
					Block.spawnAsEntity(world, pos.down(), new ItemStack(Items.GOLDEN_APPLE, 1, 1));
				}
				else if (new Random().nextInt(1024) == 0) {
					Block.spawnAsEntity(world, pos.down(), new ItemStack(Items.GOLDEN_APPLE, 1, 0));
				}
				else if (new Random().nextInt(128) == 0) {
					Block.spawnAsEntity(world, pos.down(), new ItemStack(Items.APPLE));
				}
			}
		}
	}

}
