package rainbeau.mithwoodforest.RMFBlocks;

import java.util.List;
import java.util.Random;

import net.minecraft.block.BlockSlab;
import net.minecraft.block.SoundType;
import net.minecraft.block.material.Material;
import net.minecraft.block.properties.IProperty;
import net.minecraft.block.properties.PropertyBool;
import net.minecraft.block.properties.PropertyEnum;
import net.minecraft.block.state.BlockStateContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.util.IStringSerializable;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

public abstract class BlockMithwoodSlab extends BlockSlab {
	
	public static final PropertyBool SEAMLESS = PropertyBool.create("seamless");
	public static final PropertyEnum VARIANT = PropertyEnum.create("variant", EnumType.class);

	public BlockMithwoodSlab() {
		super(Material.WOOD);
		this.setSoundType(SoundType.WOOD);
		IBlockState iblockstate = this.blockState.getBaseState();
		if (isDouble()) {
			iblockstate = iblockstate.withProperty(SEAMLESS, Boolean.valueOf(false));
		} 
		else {
			iblockstate = iblockstate.withProperty(HALF, BlockSlab.EnumBlockHalf.BOTTOM);
		}
		setDefaultState(iblockstate.withProperty(VARIANT, EnumType.MITHWOOD));
		setLightOpacity(0);
	}

	@Override
	public Item getItemDropped(IBlockState state, Random rand, int fortune) {
		return Item.getItemFromBlock(BlocksCreate.mithwood_slab_single);
	}
	  
	@SideOnly(Side.CLIENT)
	public Item getItem(World worldIn, BlockPos pos) {
		return Item.getItemFromBlock(BlocksCreate.mithwood_slab_single);
	}
	  
	@Override
	public String getUnlocalizedName(int meta) {
		return super.getUnlocalizedName() + "." + EnumType.byMetadata(meta).getUnlocalizedName();
	}
	
	@Override
	public IProperty getVariantProperty() {
		return VARIANT;
	}
	  
	public Object getVariant(ItemStack stack) {
		return EnumType.byMetadata(stack.getMetadata() & 0x7);
	}
	
	@Override
	@SideOnly(Side.CLIENT)
	public void getSubBlocks(Item itemIn, CreativeTabs tab, List list) {
		if (itemIn != Item.getItemFromBlock(BlocksCreate.mithwood_slab_double)) {
			EnumType[] aenumtype = EnumType.values();
			int i = aenumtype.length;
			for (int j = 0; j < i; j++) {
				EnumType enumtype = aenumtype[j];
				list.add(new ItemStack(itemIn, 1, enumtype.getMetadata()));
			}
		}
	}
	
	@Override
	public IBlockState getStateFromMeta(int meta) {
		IBlockState iblockstate = getDefaultState().withProperty(VARIANT, EnumType.byMetadata(meta & 0x7));
		if (isDouble()) {
			iblockstate = iblockstate.withProperty(SEAMLESS, Boolean.valueOf((meta & 0x8) != 0));
		} 
		else {
			iblockstate = iblockstate.withProperty(HALF, (meta & 0x8) == 0 ? BlockSlab.EnumBlockHalf.BOTTOM : BlockSlab.EnumBlockHalf.TOP);
		}
		return iblockstate;
	}
	
	@Override
	public int getMetaFromState(IBlockState state) {
		byte b0 = 0;
		int i = b0 | ((EnumType)state.getValue(VARIANT)).getMetadata();
		if (isDouble()) {
			if (((Boolean)state.getValue(SEAMLESS)).booleanValue()) {
				i |= 0x8;
			}
		}
		else if (state.getValue(HALF) == BlockSlab.EnumBlockHalf.TOP) {
			i |= 0x8;
		}
		return i;
	}
	
	@Override
	protected BlockStateContainer createBlockState() {
		return isDouble() ? new BlockStateContainer(this, new IProperty[] { SEAMLESS, VARIANT }) : new BlockStateContainer(this, new IProperty[] { HALF, VARIANT });
	}
	
	@Override
	public int damageDropped(IBlockState state) {
		return ((EnumType)state.getValue(VARIANT)).getMetadata();
	}
	  
	public static enum EnumType implements IStringSerializable {
		MITHWOOD(0, "mithwood_slab");
	    private static final EnumType[] META_LOOKUP;
	    private final int meta;
	    private final String name;
	    private EnumType(int meta, String name) {
	    	this.meta = meta;
	    	this.name = name;
	    }
	    public int getMetadata() {
	    	return this.meta;
	    }
	    @Override
	    public String toString() {
	    	return this.name;
	    }
	    public static EnumType byMetadata(int meta) {
	    	if ((meta < 0) || (meta >= META_LOOKUP.length)) {
	    		meta = 0;
	    	}
	    	return META_LOOKUP[meta];
	    }
	    @Override
	    public String getName() {
	    	return this.name;
	    }
	    public String getUnlocalizedName() {
	    	return this.name;
	    }
	    static {
	    	META_LOOKUP = new EnumType[values().length];
	    	EnumType[] var0 = values();
	    	int var1 = var0.length;
	    	for (int var2 = 0; var2 < var1; var2++) {
	    		EnumType var3 = var0[var2];
	    		META_LOOKUP[var3.getMetadata()] = var3;
	    	}
	    }
	}
	
}
