package rainbeau.mithwoodforest.RMFEntities;

import java.util.List;

import javax.annotation.Nullable;

import net.minecraft.block.Block;
import net.minecraft.block.state.IBlockState;
import net.minecraft.enchantment.EnchantmentHelper;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityCreature;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.EnumCreatureAttribute;
import net.minecraft.entity.IEntityLivingData;
import net.minecraft.entity.IRangedAttackMob;
import net.minecraft.entity.SharedMonsterAttributes;
import net.minecraft.entity.ai.EntityAIAttackMelee;
import net.minecraft.entity.ai.EntityAIAvoidEntity;
import net.minecraft.entity.ai.EntityAIBase;
import net.minecraft.entity.ai.EntityAIHurtByTarget;
import net.minecraft.entity.ai.EntityAILookIdle;
import net.minecraft.entity.ai.EntityAINearestAttackableTarget;
import net.minecraft.entity.ai.EntityAIOpenDoor;
import net.minecraft.entity.ai.EntityAISwimming;
import net.minecraft.entity.ai.EntityAIWander;
import net.minecraft.entity.ai.EntityAIWatchClosest;
import net.minecraft.entity.monster.EntityMob;
import net.minecraft.entity.monster.EntityPigZombie;
import net.minecraft.entity.monster.EntitySkeleton;
import net.minecraft.entity.monster.EntityZombie;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.projectile.EntityTippedArrow;
import net.minecraft.init.Enchantments;
import net.minecraft.init.Items;
import net.minecraft.init.MobEffects;
import net.minecraft.init.PotionTypes;
import net.minecraft.init.SoundEvents;
import net.minecraft.inventory.EntityEquipmentSlot;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.item.ItemSword;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.network.datasync.DataParameter;
import net.minecraft.network.datasync.DataSerializers;
import net.minecraft.network.datasync.EntityDataManager;
import net.minecraft.pathfinding.PathNavigateGround;
import net.minecraft.potion.PotionEffect;
import net.minecraft.potion.PotionType;
import net.minecraft.potion.PotionUtils;
import net.minecraft.util.EnumHand;
import net.minecraft.util.SoundEvent;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.MathHelper;
import net.minecraft.world.DifficultyInstance;
import net.minecraft.world.EnumDifficulty;
import net.minecraft.world.World;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;
import rainbeau.mithwoodforest.RMFConfig.Config;
import rainbeau.mithwoodforest.RMFItems.ItemsCreate;

public class GuardianEntity extends EntityMob implements IRangedAttackMob {

	private static final DataParameter<Boolean> SWINGING_ARMS = EntityDataManager.<Boolean>createKey(GuardianEntity.class, DataSerializers.BOOLEAN);
	private final GuardianEntityAIBow aiArrowAttack = new GuardianEntityAIBow(this, 1.0D, 15, 20.0F);
	private boolean isArcher = false;
	private boolean hasShield = false;
	private boolean isSkittish = false;
	private int armorLevel = rand.nextInt(100);
    private int potionTimer;
   	protected EntityAIBase aiAvoidPlayer = new EntityAIAvoidEntity(this, EntityPlayer.class, 6.0F, 1.0D, 1.2D); 
	protected EntityAIBase aiAvoidPigZombie = new EntityAIAvoidEntity(this, EntityPigZombie.class, 6.0F, 1.0D, 1.2D);    
	protected EntityAIBase aiAvoidSkeleton = new EntityAIAvoidEntity(this, EntitySkeleton.class, 6.0F, 1.0D, 1.2D);
	protected EntityAIBase aiAvoidZombie = new EntityAIAvoidEntity(this, EntityZombie.class, 6.0F, 1.0D, 1.2D);    
	
    public GuardianEntity(World worldIn) {
        super(worldIn);
        ((PathNavigateGround)this.getNavigator()).setBreakDoors(true);
        this.setCombatTask();
    }

    @Override
    protected void initEntityAI() {
        this.tasks.addTask(1, new EntityAISwimming(this));
        this.tasks.addTask(2, new EntityAIOpenDoor(this, true));
        this.tasks.addTask(3, new EntityAIWander(this, 1.0D));
        this.tasks.addTask(4, new EntityAIWatchClosest(this, EntityPlayer.class, 8.0F));
        this.tasks.addTask(4, new EntityAILookIdle(this));
        this.targetTasks.addTask(1, new EntityAIHurtByTarget(this, false, new Class[0]));
        this.targetTasks.addTask(2, new EntityAINearestAttackableTarget(this, EntityPlayer.class, true));
        this.targetTasks.addTask(3, new EntityAINearestAttackableTarget(this, EntityPigZombie.class, true));
        this.targetTasks.addTask(3, new EntityAINearestAttackableTarget(this, EntitySkeleton.class, true));
        this.targetTasks.addTask(3, new EntityAINearestAttackableTarget(this, EntityZombie.class, true));
    }

    @Override
    protected void applyEntityAttributes() {
        super.applyEntityAttributes();
        this.getEntityAttribute(SharedMonsterAttributes.MOVEMENT_SPEED).setBaseValue(0.3D);
        this.getEntityAttribute(SharedMonsterAttributes.ATTACK_DAMAGE).setBaseValue(4.0D);
		this.getEntityAttribute(SharedMonsterAttributes.MAX_HEALTH).setBaseValue(30.0D);
    }
 
    @Override
    protected void entityInit() {
        super.entityInit();
        this.dataManager.register(SWINGING_ARMS, Boolean.valueOf(false));
    }

    @Override
    protected SoundEvent getAmbientSound() {
    	return SoundEvents.ENTITY_VILLAGER_AMBIENT;
    }

    @Override
    protected SoundEvent getHurtSound() {
    	return SoundEvents.ENTITY_VILLAGER_HURT;
    }

    @Override
    protected SoundEvent getDeathSound() {
    	return SoundEvents.ENTITY_VILLAGER_DEATH;
    }

    @Override
    protected void playStepSound(BlockPos pos, Block blockIn) {
        SoundEvent soundevent = SoundEvents.ENTITY_ZOMBIE_VILLAGER_STEP;
        this.playSound(soundevent, 0.15F, 1.0F);
    }

	@Override
	public boolean getCanSpawnHere() {
		if (Config.AddGuardians < 1) { return false; }
		BlockPos blockpos = new BlockPos(this.posX, this.getEntityBoundingBox().minY, this.posZ);
        if (blockpos.getY() < 32) {
            return false;
        }
       	int light = this.worldObj.getLightFromNeighbors(blockpos);
       	if (light < 6) {
       		if (rand.nextInt(4) > 0) {
       			return false;
       		}
       	}
       	if (this.worldObj.getDifficulty() == EnumDifficulty.PEACEFUL) {
       		return false;
       	}
       	IBlockState iblockstate = this.worldObj.getBlockState((new BlockPos(this)).down());
       	return iblockstate.func_189884_a(this);
	}
  
	@Override
	protected boolean canDespawn() {
		return true;
	}

	private final EntityAIAttackMelee aiAttackOnCollide = new EntityAIAttackMelee(this, 1.2D, true) {
		@Override
		public void resetTask() {
			super.resetTask();
            GuardianEntity.this.setSwingingArms(false);
        }
		@Override
		public void startExecuting() {
            super.startExecuting();
            GuardianEntity.this.setSwingingArms(true);
        }
    };

    @Override
    public boolean attackEntityAsMob(Entity entityIn) {
        if (super.attackEntityAsMob(entityIn)) { 
        	if (!this.isArcher && (entityIn instanceof EntityLivingBase)) {
        		int swordEffect = rand.nextInt(10);
        		if (swordEffect < 3 && !((EntityLivingBase)entityIn).isPotionActive(MobEffects.SLOWNESS)) {
        			((EntityLivingBase)entityIn).addPotionEffect(new PotionEffect(MobEffects.SLOWNESS, 600));
        		}
        		else if (swordEffect < 4 && !((EntityLivingBase)entityIn).isPotionActive(MobEffects.POISON)) {
        			((EntityLivingBase)entityIn).addPotionEffect(new PotionEffect(MobEffects.POISON, 225));
        		}
        		else if (swordEffect < 5 && !((EntityLivingBase)entityIn).isPotionActive(MobEffects.WEAKNESS)) {
        			((EntityLivingBase)entityIn).addPotionEffect(new PotionEffect(MobEffects.WEAKNESS, 600));
        		}
        	}
            return true;
        }
        else {
        	return false;
        }
    }

    @Override
    public EnumCreatureAttribute getCreatureAttribute() {
        return EnumCreatureAttribute.UNDEFINED;
    }

    public void onLivingUpdate() {
    	if (this.worldObj.isRemote) {
    		this.setSize(0.6F, 1.99F);
    	}
		if (!this.isSkittish && (this.getHealth() < (this.getMaxHealth()/6))) {
			this.isSkittish = true;
            this.tasks.removeTask(this.aiAttackOnCollide);
            this.tasks.removeTask(this.aiArrowAttack);
           	this.tasks.addTask(2, aiAvoidPlayer); 
           	this.tasks.addTask(2, aiAvoidPigZombie); 
           	this.tasks.addTask(2, aiAvoidSkeleton); 
           	this.tasks.addTask(2, aiAvoidZombie); 
		}
    	if (this.isSkittish && (this.getHealth() > (this.getMaxHealth()/2))) {
    		this.isSkittish = false;
           	this.tasks.removeTask(aiAvoidPlayer); 
           	this.tasks.removeTask(aiAvoidPigZombie); 
           	this.tasks.removeTask(aiAvoidSkeleton); 
           	this.tasks.removeTask(aiAvoidZombie); 
    		setCombatTask();
    	}
    	this.potionTimer--;
    	if (this.potionTimer < -40) {
    		if (this.isSkittish) {
    			if (rand.nextInt(20) == 0) {
       				PotionType potiontype = PotionTypes.HEALING;
        			this.setItemStackToSlot(EntityEquipmentSlot.OFFHAND, PotionUtils.addPotionToItemStack(new ItemStack(Items.POTIONITEM), potiontype));
        			this.potionTimer = this.getHeldItemOffhand().getMaxItemUseDuration();
        			this.worldObj.playSound((EntityPlayer)null, this.posX, this.posY, this.posZ, SoundEvents.ENTITY_WITCH_DRINK, this.getSoundCategory(), 1.0F, 0.8F + this.rand.nextFloat() * 0.4F);
    			}
    			else if ((rand.nextInt(40) == 0) && (!this.isPotionActive(MobEffects.REGENERATION))) {
    				PotionType potiontype = PotionTypes.REGENERATION;
    				this.setItemStackToSlot(EntityEquipmentSlot.OFFHAND, PotionUtils.addPotionToItemStack(new ItemStack(Items.POTIONITEM), potiontype));
    				this.potionTimer = this.getHeldItemOffhand().getMaxItemUseDuration();
    				this.worldObj.playSound((EntityPlayer)null, this.posX, this.posY, this.posZ, SoundEvents.ENTITY_WITCH_DRINK, this.getSoundCategory(), 1.0F, 0.8F + this.rand.nextFloat() * 0.4F);
    			}
    		}
    	}
    	else if (this.potionTimer == 0) {
    		ItemStack itemstack = this.getHeldItemOffhand();
    		this.setItemStackToSlot(EntityEquipmentSlot.OFFHAND, (ItemStack)null);
    		if (itemstack != null && itemstack.getItem() == Items.POTIONITEM) {
    			List<PotionEffect> list = PotionUtils.getEffectsFromStack(itemstack);
    			if (list != null) {
    				for (PotionEffect potioneffect : list) {
    					this.addPotionEffect(new PotionEffect(potioneffect));
    				}
    			}
    		}
    		if (this.hasShield) {
    			setItemStackToSlot(EntityEquipmentSlot.OFFHAND, new ItemStack(Items.SHIELD, 1));
    		}
        }
        super.onLivingUpdate();
    }
    
    @Override
    public void updateRidden() {
        super.updateRidden();
        if (this.getRidingEntity() instanceof EntityCreature) {
            EntityCreature entitycreature = (EntityCreature)this.getRidingEntity();
            this.renderYawOffset = entitycreature.renderYawOffset;
        }
    }

    @Override
    protected void setEquipmentBasedOnDifficulty(DifficultyInstance difficulty) {
        super.setEquipmentBasedOnDifficulty(difficulty);
        this.setItemStackToSlot(EntityEquipmentSlot.MAINHAND, new ItemStack(Items.BOW));
    }

    @Override
    @Nullable
    public IEntityLivingData onInitialSpawn(DifficultyInstance difficulty, @Nullable IEntityLivingData livingdata) {
        livingdata = super.onInitialSpawn(difficulty, livingdata);
        if (rand.nextInt(2) == 0) {
            this.tasks.addTask(2, this.aiArrowAttack);
            this.setEquipmentBasedOnDifficulty(difficulty);
            this.setEnchantmentBasedOnDifficulty(difficulty);
            isArcher = true;
        }
		addRandomArmor();
        this.setCanPickUpLoot(this.rand.nextFloat() < 0.55F * difficulty.getClampedAdditionalDifficulty());
        return livingdata;
    }

    public void setCombatTask() {
        if (this.worldObj != null && !this.worldObj.isRemote) {
            this.tasks.removeTask(this.aiAttackOnCollide);
            this.tasks.removeTask(this.aiArrowAttack);
            ItemStack itemstack = this.getHeldItemMainhand();
            if (itemstack != null && itemstack.getItem() == Items.BOW) {
                int i = 15;
                if (this.worldObj.getDifficulty() != EnumDifficulty.HARD) {
                    i = 30;
                }
                this.aiArrowAttack.setAttackCooldown(i);
                this.tasks.addTask(2, this.aiArrowAttack);
            }
            else if (itemstack != null && itemstack.getItem() instanceof ItemSword) {
               this.tasks.addTask(2, this.aiAttackOnCollide);
            }
        }
    }

    @Override
    public void attackEntityWithRangedAttack(EntityLivingBase target, float arrowVelocity) {
       	EntityTippedArrow entitytippedarrow = new EntityTippedArrow(this.worldObj, this);
       	double d0 = target.posX - this.posX;
       	double d1 = target.getEntityBoundingBox().minY + (double)(target.height / 3.0F) - entitytippedarrow.posY;
       	double d2 = target.posZ - this.posZ;
       	double d3 = (double)MathHelper.sqrt_double(d0 * d0 + d2 * d2);
       	entitytippedarrow.setThrowableHeading(d0, d1 + d3 * 0.20000000298023224D, d2, 1.6F, (float)(14 - this.worldObj.getDifficulty().getDifficultyId() * 4));
       	int i = EnchantmentHelper.getMaxEnchantmentLevel(Enchantments.POWER, this);
       	int j = EnchantmentHelper.getMaxEnchantmentLevel(Enchantments.PUNCH, this);
       	DifficultyInstance difficultyinstance = this.worldObj.getDifficultyForLocation(new BlockPos(this));
       	entitytippedarrow.setDamage((double)(arrowVelocity * 2.0F) + this.rand.nextGaussian() * 0.25D + (double)((float)this.worldObj.getDifficulty().getDifficultyId() * 0.11F));
       	if (i > 0) {
       		entitytippedarrow.setDamage(entitytippedarrow.getDamage() + (double)i * 0.5D + 0.5D);
       	}
       	if (j > 0) {
       		entitytippedarrow.setKnockbackStrength(j);
       	}
       	boolean flag = this.isBurning() && difficultyinstance.func_190083_c() && this.rand.nextBoolean();
       	flag = flag || EnchantmentHelper.getMaxEnchantmentLevel(Enchantments.FLAME, this) > 0;
       	if (flag) {
       		entitytippedarrow.setFire(100);
       	}
       	ItemStack itemstack = this.getHeldItem(EnumHand.OFF_HAND);
       	if (itemstack != null && itemstack.getItem() == Items.TIPPED_ARROW) {
       		entitytippedarrow.setPotionEffect(itemstack);
       	}
       	else {
       		int arrowEffect = rand.nextInt(10);
       		if (arrowEffect < 3 && !target.isPotionActive(MobEffects.SLOWNESS)) {
       			entitytippedarrow.addEffect(new PotionEffect(MobEffects.SLOWNESS, 600));
       		}
       		else if (arrowEffect < 4 && !target.isPotionActive(MobEffects.POISON)) {
       			entitytippedarrow.addEffect(new PotionEffect(MobEffects.POISON, 225));
       		}
       		else if (arrowEffect < 5 && !target.isPotionActive(MobEffects.WEAKNESS)) {
       			entitytippedarrow.addEffect(new PotionEffect(MobEffects.WEAKNESS, 600));
       		}
       	}
       	this.playSound(SoundEvents.ENTITY_SKELETON_SHOOT, 1.0F, 1.0F / (this.getRNG().nextFloat() * 0.4F + 0.8F));
       	this.worldObj.spawnEntityInWorld(entitytippedarrow);
    }

    @Override
    public void setItemStackToSlot(EntityEquipmentSlot slotIn, @Nullable ItemStack stack) {
        super.setItemStackToSlot(slotIn, stack);
        if (!this.worldObj.isRemote && slotIn == EntityEquipmentSlot.MAINHAND) {
            this.setCombatTask();
        }
    }

    @Override
    public float getEyeHeight() {
        return 1.74F;
    }
 
    @Override
    public double getYOffset() {
        return -0.35D;
    }

    @SideOnly(Side.CLIENT)
    public boolean isSwingingArms() {
        return ((Boolean)this.dataManager.get(SWINGING_ARMS)).booleanValue();
    }

    public void setSwingingArms(boolean swingingArms)
    {
        this.dataManager.set(SWINGING_ARMS, Boolean.valueOf(swingingArms));
    }

	private void addRandomArmor() {
		if (armorLevel > 94) {
			if (Config.EmeraldArmor == true) { armorLevel = 6; }
			else { armorLevel = 5; }
		}
		else if (armorLevel > 84) { armorLevel = 5; }
		else if (armorLevel > 69) {
			if (Config.ObsidianArmor == true) { armorLevel = 4; }
			else { armorLevel = 2; }
		}
		else if (armorLevel > 54) { 
			if (Config.LapisArmor == true) { armorLevel = 3; }
			else { armorLevel = 2; }
		}
		else if (armorLevel > 24) { armorLevel = 2; }
		else { armorLevel = 1; }
		for(EntityEquipmentSlot slot : EntityEquipmentSlot.values()) {
			ItemStack itemStack = getItemStackFromSlot(slot);
			if(itemStack == null && (rand.nextInt(5) > 0)) {
				Item item = getArmorByLevel(slot, armorLevel);
				if(item != null) {
					ItemStack stack = new ItemStack(item);
					setItemStackToSlot(slot, stack);
				}
			}
		}
		if(!isArcher) {
			setItemStackToSlot(EntityEquipmentSlot.MAINHAND, getSwordForLevel(armorLevel));
			int shieldChance = rand.nextInt(10) + armorLevel;
			if(shieldChance > 7) {
				setItemStackToSlot(EntityEquipmentSlot.OFFHAND, new ItemStack(Items.SHIELD, 1));
				this.hasShield = true;
			}
		} 
	}

	private ItemStack getSwordForLevel(int armorLevel) {
		if (armorLevel < 3) { return new ItemStack(Items.IRON_SWORD); }
		else if (armorLevel == 3) { return new ItemStack(ItemsCreate.lapis_sword); }
		else if (armorLevel == 4) { return new ItemStack(ItemsCreate.obsidian_sword); }
		else if (armorLevel == 5) { return new ItemStack(Items.DIAMOND_SWORD); }
		else { return new ItemStack(ItemsCreate.emerald_sword); }
	}
  
    public static Item getArmorByLevel(EntityEquipmentSlot slotIn, int level) {
        switch (slotIn) {
            case HEAD:
                if (level == 1) { return Items.CHAINMAIL_HELMET; }
                else if (level == 2) { return Items.IRON_HELMET; }
                else if (level == 3) { return ItemsCreate.lapis_helmet; }
                else if (level == 4) { return ItemsCreate.obsidian_helmet; }
                else if (level == 5) { return Items.DIAMOND_HELMET; }
                else if (level == 6) { return ItemsCreate.emerald_helmet; }
            case CHEST:
                if (level == 1) { return Items.CHAINMAIL_CHESTPLATE; }
                else if (level == 2) { return Items.IRON_CHESTPLATE; }
                else if (level == 3) { return ItemsCreate.lapis_chestplate; }
                else if (level == 4) { return ItemsCreate.obsidian_chestplate; }
                else if (level == 5) { return Items.DIAMOND_CHESTPLATE; }
                else if (level == 6) { return ItemsCreate.emerald_chestplate; }
            case LEGS:
                if (level == 1) { return Items.CHAINMAIL_LEGGINGS; }
                else if (level == 2) { return Items.IRON_LEGGINGS; }
                else if (level == 3) { return ItemsCreate.lapis_leggings; }
                else if (level == 4) { return ItemsCreate.obsidian_leggings; }
                else if (level == 5) { return Items.DIAMOND_LEGGINGS; }
                else if (level == 6) { return ItemsCreate.emerald_leggings; }
            case FEET:
                if (level == 1) { return Items.CHAINMAIL_BOOTS; }
                else if (level == 2) { return Items.IRON_BOOTS; }
                else if (level == 3) { return ItemsCreate.lapis_boots; }
                else if (level == 4) { return ItemsCreate.obsidian_boots; }
                else if (level == 5) { return Items.DIAMOND_BOOTS; }
                else if (level == 6) { return ItemsCreate.emerald_boots; }
            default:
                return null;
        }
    }
	
    @Override
    public void writeEntityToNBT(NBTTagCompound root) {
    	super.writeEntityToNBT(root);
    	root.setBoolean("isArcher", this.isArcher);
    	root.setBoolean("hasShield", this.hasShield);
    }
    
    @Override
    public void readEntityFromNBT(NBTTagCompound root) {
    	super.readEntityFromNBT(root);
    	isArcher = root.getBoolean("isArcher");
    	hasShield = root.getBoolean("hasShield");
    }
 
    @Override
    protected void dropFewItems(boolean hitByPlayer, int lootingLevel) {
   		if (this.isArcher) { dropItem(Items.ARROW, rand.nextInt(3)); } 
    }

}
