package rainbeau.mithwoodforest.RMFItems;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Stack;

import com.google.common.collect.Sets;

import net.minecraft.block.Block;
import net.minecraft.block.BlockLog;
import net.minecraft.block.BlockNewLog;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Blocks;
import net.minecraft.item.Item;
import net.minecraft.item.ItemAxe;
import net.minecraft.item.ItemStack;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;
import net.minecraftforge.fml.common.registry.GameRegistry;

public class ItemAxeMithril extends ItemAxe {
	
	private static final Set<Block> EFFECTIVE_ON = Sets.newHashSet(new Block[] { Blocks.PLANKS, Blocks.BOOKSHELF, Blocks.LOG, Blocks.LOG2, Blocks.CHEST, Blocks.PUMPKIN, Blocks.LIT_PUMPKIN, Blocks.MELON_BLOCK, Blocks.LADDER, Blocks.WOODEN_BUTTON, Blocks.WOODEN_PRESSURE_PLATE });

	public ItemAxeMithril(String unlocalizedName, Item.ToolMaterial material, float damage, float speed) {
	    super(material, damage, speed);
	    this.setUnlocalizedName(unlocalizedName);
	    this.setRegistryName(unlocalizedName);
	    GameRegistry.register(this);
	}
	
	@Override
	public boolean onBlockStartBreak(ItemStack stack, BlockPos blockPos, EntityPlayer player) {
		World world = player.worldObj;
		IBlockState firstblock = world.getBlockState(blockPos);
		if (firstblock.getBlock() instanceof BlockLog || firstblock.getBlock() instanceof BlockNewLog) {
			List<BlockPos> logBlocks = getConnectedLogBlocks(world, blockPos);
			for (BlockPos pos : logBlocks) {
				if (!pos.equals(blockPos)) {
					IBlockState block = world.getBlockState(pos);
					if (block.getBlock() instanceof BlockLog || block.getBlock() instanceof BlockNewLog) {
						world.setBlockToAir(pos);
						block.getBlock().dropBlockAsItem(world, pos, block, 0);
					}
				}
			}
		}
		stack.damageItem(1, player);
		return false;
	}
  
	@Override
	public boolean onBlockDestroyed(ItemStack stack, World worldIn, IBlockState state, BlockPos pos, EntityLivingBase entityLiving) {
		return true;
	}
  
	private List<BlockPos> getConnectedLogBlocks(World world, BlockPos blockPos) {
		List<BlockPos> res = new ArrayList();
		Stack<BlockPos> positions = new Stack();
		positions.push(blockPos);
        IBlockState start = world.getBlockState(blockPos);
        Block startBlock = start.getBlock();
        do {
        	BlockPos pos = (BlockPos)positions.pop();
        	res.add(pos);
        	if (pos.distanceSq(blockPos) < 4096.0D) { // find all connected logs within 64 blocks
        		for (BlockPos newPos : BlockPos.getAllInBox(new BlockPos(pos.getX() - 1, pos.getY() - 1, pos.getZ() - 1), new BlockPos(pos.getX() + 1, pos.getY() + 1, pos.getZ() + 1))) {
        			boolean isLog = isValidLogBlock(world, newPos, startBlock);
        			if ((isLog) && (!positions.contains(newPos)) && (!res.contains(newPos))) {
        				positions.push(newPos);
        			}
        		}
        	}
        } 
        while (!positions.empty());
        return res;
	}
  
	private boolean isValidLogBlock(World world, BlockPos pos, Block start) {
		IBlockState state = world.getBlockState(pos);
		Block block = state.getBlock();
		if (start != null) {
			return block == start;
		}
		return (block != null) && (block instanceof BlockLog || block instanceof BlockNewLog);
	}
 
	@Override
	public boolean getIsRepairable(ItemStack toRepair, ItemStack repair) {
		if (toRepair.getItem().equals(ItemsCreate.mithril_axe)) {
			if (repair.getItem().equals(ItemsCreate.mithril_ingot)) {
				return true;
			}
		}
		return false;
	}

}
