package rainbeau.mithwoodforest.RMFItems;

import net.minecraft.block.Block;
import net.minecraft.block.BlockSlab;
import net.minecraft.block.SoundType;
import net.minecraft.block.properties.IProperty;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemBlock;
import net.minecraft.item.ItemStack;
import net.minecraft.util.EnumActionResult;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumHand;
import net.minecraft.util.SoundCategory;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;
import rainbeau.mithwoodforest.RMFBlocks.BlocksCreate;

public class ItemMithwoodSlab extends ItemBlock {
	
	private final BlockSlab singleSlab;
	private final BlockSlab doubleSlab;
	  
	public ItemMithwoodSlab(Block block) {
		super(block);
		this.singleSlab = ((BlockSlab)BlocksCreate.mithwood_slab_single);
		this.doubleSlab = ((BlockSlab)BlocksCreate.mithwood_slab_double);
		setMaxDamage(0);
		setHasSubtypes(true);
	}
	 
	@Override
	public int getMetadata(int damage) {
		return damage;
	}
	
	@Override
	public String getUnlocalizedName(ItemStack stack) {
		return this.singleSlab.getUnlocalizedName(stack.getMetadata());
	}
	
	@Override
	public EnumActionResult onItemUse(ItemStack stack, EntityPlayer playerIn, World worldIn, BlockPos pos, EnumHand hand, EnumFacing facing, float hitX, float hitY, float hitZ) {
		if ((stack.stackSize != 0) && (playerIn.canPlayerEdit(pos.offset(facing), facing, stack))) {
			Comparable<?> comparable = this.singleSlab.getTypeForItem(stack);
			IBlockState iblockstate = worldIn.getBlockState(pos);
			if (iblockstate.getBlock() == this.singleSlab) {
				IProperty<?> iproperty = this.singleSlab.getVariantProperty();
				Comparable<?> comparable1 = iblockstate.getValue(iproperty);
				BlockSlab.EnumBlockHalf blockslab$enumblockhalf = (BlockSlab.EnumBlockHalf)iblockstate.getValue(BlockSlab.HALF);
				if (((facing == EnumFacing.UP) && (blockslab$enumblockhalf == BlockSlab.EnumBlockHalf.BOTTOM)) || ((facing == EnumFacing.DOWN) && (blockslab$enumblockhalf == BlockSlab.EnumBlockHalf.TOP) && (comparable1 == comparable))) {
					IBlockState iblockstate1 = makeState(iproperty, comparable1);
					AxisAlignedBB axisalignedbb = iblockstate1.getSelectedBoundingBox(worldIn, pos);
					if ((axisalignedbb != Block.NULL_AABB) && (worldIn.checkNoEntityCollision(axisalignedbb.offset(pos))) && (worldIn.setBlockState(pos, iblockstate1, 11))) {
						SoundType soundtype = this.doubleSlab.getSoundType();
						worldIn.playSound(playerIn, pos, soundtype.getPlaceSound(), SoundCategory.BLOCKS, (soundtype.getVolume() + 1.0F) / 2.0F, soundtype.getPitch() * 0.8F);
						stack.stackSize -= 1;
					}	
					return EnumActionResult.SUCCESS;
				}
			}
			return tryPlace(playerIn, stack, worldIn, pos.offset(facing), comparable) ? EnumActionResult.SUCCESS : super.onItemUse(stack, playerIn, worldIn, pos, hand, facing, hitX, hitY, hitZ);
		}
		return EnumActionResult.FAIL;
	}
	
	@Override
	@SideOnly(Side.CLIENT)
	public boolean canPlaceBlockOnSide(World worldIn, BlockPos pos, EnumFacing side, EntityPlayer player, ItemStack stack) {
		BlockPos blockpos = pos;
	    IProperty<?> iproperty = this.singleSlab.getVariantProperty();
	    Comparable<?> comparable = this.singleSlab.getTypeForItem(stack);
	    IBlockState iblockstate = worldIn.getBlockState(pos);
	    if (iblockstate.getBlock() == this.singleSlab) {
	    	boolean flag = iblockstate.getValue(BlockSlab.HALF) == BlockSlab.EnumBlockHalf.TOP;
	    	if (((side == EnumFacing.UP) && (!flag)) || ((side == EnumFacing.DOWN) && (flag) && (comparable == iblockstate.getValue(iproperty)))) {
	    		return true;
	    	}
	    }
	    pos = pos.offset(side);
	    IBlockState iblockstate1 = worldIn.getBlockState(pos);
	    return (iblockstate1.getBlock() == this.singleSlab) && (comparable == iblockstate1.getValue(iproperty)) ? true : super.canPlaceBlockOnSide(worldIn, blockpos, side, player, stack);
	}
	  
	private boolean tryPlace(EntityPlayer player, ItemStack stack, World worldIn, BlockPos pos, Object itemSlabType) {
		IBlockState iblockstate = worldIn.getBlockState(pos);
	    if (iblockstate.getBlock() == this.singleSlab) {
	    	Comparable<?> comparable = iblockstate.getValue(this.singleSlab.getVariantProperty());
	    	if (comparable == itemSlabType) {
	    		IBlockState iblockstate1 = makeState(this.singleSlab.getVariantProperty(), comparable);
	    		AxisAlignedBB axisalignedbb = iblockstate1.getSelectedBoundingBox(worldIn, pos);
	    		if ((axisalignedbb != Block.NULL_AABB) && (worldIn.checkNoEntityCollision(axisalignedbb.offset(pos))) && (worldIn.setBlockState(pos, iblockstate1, 11))) {
	    			SoundType soundtype = this.doubleSlab.getSoundType();
	    			worldIn.playSound(player, pos, soundtype.getPlaceSound(), SoundCategory.BLOCKS, (soundtype.getVolume() + 1.0F) / 2.0F, soundtype.getPitch() * 0.8F);
	    			stack.stackSize -= 1;
	    		}
	    		return true;
	    	}
	    }
	    return false;
	}
	  
	protected IBlockState makeState(IProperty p_185055_1_, Comparable p_185055_2_) {
		return this.doubleSlab.getDefaultState().withProperty(p_185055_1_, p_185055_2_);
	}

}
