package rainbeau.mithwoodforest.RMFItems;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Stack;

import com.google.common.collect.Sets;

import net.minecraft.block.Block;
import net.minecraft.block.BlockDirt;
import net.minecraft.block.BlockLog;
import net.minecraft.block.BlockNewLog;
import net.minecraft.block.BlockOre;
import net.minecraft.block.BlockRedstoneOre;
import net.minecraft.block.material.Material;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Blocks;
import net.minecraft.init.SoundEvents;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.item.ItemTool;
import net.minecraft.util.EnumActionResult;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumHand;
import net.minecraft.util.SoundCategory;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;
import net.minecraftforge.fml.common.registry.GameRegistry;
import rainbeau.mithwoodforest.RMFBlocks.BlocksCreate;

public class ItemPaxelMithril extends ItemTool {

	private static final Set<Block> EFFECTIVE_ON = Sets.newHashSet(new Block[] {Blocks.ACTIVATOR_RAIL, Blocks.COAL_ORE, Blocks.COBBLESTONE, Blocks.DETECTOR_RAIL, Blocks.DIAMOND_BLOCK, Blocks.DIAMOND_ORE, Blocks.DOUBLE_STONE_SLAB, Blocks.GOLDEN_RAIL, Blocks.GOLD_BLOCK, Blocks.GOLD_ORE, Blocks.ICE, Blocks.IRON_BLOCK, Blocks.IRON_ORE, Blocks.LAPIS_BLOCK, Blocks.LAPIS_ORE, Blocks.LIT_REDSTONE_ORE, Blocks.MOSSY_COBBLESTONE, Blocks.NETHERRACK, Blocks.PACKED_ICE, Blocks.RAIL, Blocks.REDSTONE_ORE, Blocks.SANDSTONE, Blocks.RED_SANDSTONE, Blocks.STONE, Blocks.STONE_SLAB, Blocks.STONE_BUTTON, Blocks.STONE_PRESSURE_PLATE,
		Blocks.PLANKS, Blocks.BOOKSHELF, Blocks.LOG, Blocks.LOG2, Blocks.CHEST, Blocks.PUMPKIN, Blocks.LIT_PUMPKIN, Blocks.MELON_BLOCK, Blocks.LADDER, Blocks.WOODEN_BUTTON, Blocks.WOODEN_PRESSURE_PLATE,
		Blocks.CLAY, Blocks.DIRT, Blocks.FARMLAND, Blocks.GRASS, Blocks.GRAVEL, Blocks.MYCELIUM, Blocks.SAND, Blocks.SNOW, Blocks.SNOW_LAYER, Blocks.SOUL_SAND, Blocks.GRASS_PATH});

	protected ItemPaxelMithril(String unlocalizedName, Item.ToolMaterial material) {
		super(1.5F, -3.0F, material, EFFECTIVE_ON);
	    this.maxStackSize = 1;
	    int durability = (int)(material.getMaxUses()*1.5);
	    setMaxDamage(durability);
	    this.setUnlocalizedName(unlocalizedName);
	    this.setRegistryName(unlocalizedName);
	    GameRegistry.register(this);
	}

	@Override
	public boolean canHarvestBlock(IBlockState blockIn) {
		Block block = blockIn.getBlock();
		if (block == BlocksCreate.unobtainium_ore) {
			return this.toolMaterial.getHarvestLevel() >= 4;
		}
		else if (block == Blocks.OBSIDIAN) {
			return this.toolMaterial.getHarvestLevel() >= 3;
		}
		else if (block != Blocks.DIAMOND_BLOCK && block != Blocks.DIAMOND_ORE) {
			if (block != Blocks.EMERALD_ORE && block != Blocks.EMERALD_BLOCK) {
				if (block != Blocks.GOLD_BLOCK && block != Blocks.GOLD_ORE) {
					if (block != Blocks.IRON_BLOCK && block != Blocks.IRON_ORE) {
						if (block != Blocks.LAPIS_BLOCK && block != Blocks.LAPIS_ORE) {
							if (block != Blocks.REDSTONE_ORE && block != Blocks.LIT_REDSTONE_ORE) {
								Material material = blockIn.getMaterial();
								return material == Material.ROCK ? true : (material == Material.IRON ? true : 
									(material == Material.ANVIL ? true : (material == Material.WOOD ? true :
									(material == Material.PLANTS ? true : (material == Material.VINE ? true :
									(block == Blocks.SNOW_LAYER ? true : (block == Blocks.SNOW)))))));
							}
							else {
								return this.toolMaterial.getHarvestLevel() >= 2;
							}
						}
						else {
							return this.toolMaterial.getHarvestLevel() >= 1;
						}
					}
					else {
						return this.toolMaterial.getHarvestLevel() >= 1;
					}
				}
				else {
					return this.toolMaterial.getHarvestLevel() >= 2;
				}
			}
			else {
				return this.toolMaterial.getHarvestLevel() >= 2;
			}
		}
		else {
			return this.toolMaterial.getHarvestLevel() >= 2;
		}
	}

	@Override
	public float getStrVsBlock(ItemStack stack, IBlockState state) {
		Material material = state.getMaterial();
		return material != Material.IRON && material != Material.ANVIL && material != Material.ROCK 
			&& material != Material.WOOD && material != Material.PLANTS && material != Material.VINE ? super.getStrVsBlock(stack, state) : this.efficiencyOnProperMaterial;
	}

	@Override
    public EnumActionResult onItemUse(ItemStack stack, EntityPlayer playerIn, World worldIn, BlockPos pos, EnumHand hand, EnumFacing facing, float hitX, float hitY, float hitZ) {
		if (!playerIn.canPlayerEdit(pos.offset(facing), facing, stack)) {
			return EnumActionResult.FAIL;
        }
        else {
        	IBlockState iblockstate = worldIn.getBlockState(pos);
            Block block = iblockstate.getBlock();
            if (facing != EnumFacing.DOWN && worldIn.getBlockState(pos.up()).getMaterial() == Material.AIR && block == Blocks.GRASS) {
            	IBlockState iblockstate1 = Blocks.GRASS_PATH.getDefaultState();
                worldIn.playSound(playerIn, pos, SoundEvents.ITEM_SHOVEL_FLATTEN, SoundCategory.BLOCKS, 1.0F, 1.0F);
                if (!worldIn.isRemote) {
                	worldIn.setBlockState(pos, iblockstate1, 11);
                    stack.damageItem(1, playerIn);
                }
                return EnumActionResult.SUCCESS;
            }
            else {
            	int hook = net.minecraftforge.event.ForgeEventFactory.onHoeUse(stack, playerIn, worldIn, pos);
            	if (hook != 0) return hook > 0 ? EnumActionResult.SUCCESS : EnumActionResult.FAIL;
            	if (facing != EnumFacing.DOWN && worldIn.isAirBlock(pos.up())) {
            		if (block == Blocks.GRASS_PATH) {
            			this.setBlock(stack, playerIn, worldIn, pos, Blocks.FARMLAND.getDefaultState());
            			return EnumActionResult.SUCCESS;
            		}
            		if (block == Blocks.DIRT) {
            			switch ((BlockDirt.DirtType)iblockstate.getValue(BlockDirt.VARIANT)) {
            				case DIRT:
            					this.setBlock(stack, playerIn, worldIn, pos, Blocks.FARMLAND.getDefaultState());
            					return EnumActionResult.SUCCESS;
            				case COARSE_DIRT:
            					this.setBlock(stack, playerIn, worldIn, pos, Blocks.DIRT.getDefaultState().withProperty(BlockDirt.VARIANT, BlockDirt.DirtType.DIRT));
            					return EnumActionResult.SUCCESS;
            				case PODZOL:
            					return EnumActionResult.PASS;
            			}
            		}
            	}
            	return EnumActionResult.PASS;
            }
        }
	}

    protected void setBlock(ItemStack stack, EntityPlayer player, World worldIn, BlockPos pos, IBlockState state) {
    	worldIn.playSound(player, pos, SoundEvents.ITEM_HOE_TILL, SoundCategory.BLOCKS, 1.0F, 1.0F);
        if (!worldIn.isRemote) {
        	worldIn.setBlockState(pos, state, 11);
            stack.damageItem(1, player);
        }
    }
    
	@Override
	public boolean onBlockStartBreak(ItemStack stack, BlockPos blockPos, EntityPlayer player) {
		World world = player.worldObj;
		IBlockState firstblock = world.getBlockState(blockPos);
		if (firstblock.getBlock() instanceof BlockLog || firstblock.getBlock() instanceof BlockNewLog) {
			List<BlockPos> logBlocks = getConnectedLogBlocks(world, blockPos);
			for (BlockPos pos : logBlocks) {
				if (!pos.equals(blockPos)) {
					IBlockState block = world.getBlockState(pos);
					if (block.getBlock() instanceof BlockLog || block.getBlock() instanceof BlockNewLog) {
						world.setBlockToAir(pos);
						block.getBlock().dropBlockAsItem(world, pos, block, 0);
					}
				}
			}
		}
		if (firstblock.getBlock() instanceof BlockOre || firstblock.getBlock() instanceof BlockRedstoneOre) {
			List<BlockPos> oreBlocks = getConnectedOreBlocks(world, blockPos);
			for (BlockPos pos : oreBlocks) {
				if (!pos.equals(blockPos)) {
					IBlockState block = world.getBlockState(pos);
					if (block.getBlock() instanceof BlockOre || block.getBlock() instanceof BlockRedstoneOre) {
						world.setBlockToAir(pos);
						block.getBlock().dropBlockAsItem(world, pos, block, 0);
					}
				}
			}
		}
		stack.damageItem(1, player);
		return false;
	}
  	
	@Override
	public boolean onBlockDestroyed(ItemStack stack, World worldIn, IBlockState state, BlockPos pos, EntityLivingBase entityLiving) {
		return true;
	}
  
	private List<BlockPos> getConnectedLogBlocks(World world, BlockPos blockPos) {
		List<BlockPos> res = new ArrayList();
		Stack<BlockPos> positions = new Stack();
		positions.push(blockPos);
        IBlockState start = world.getBlockState(blockPos);
        Block startBlock = start.getBlock();
        do {
        	BlockPos pos = (BlockPos)positions.pop();
        	res.add(pos);
        	if (pos.distanceSq(blockPos) < 4096.0D) { // find all connected logs within 64 blocks
        		for (BlockPos newPos : BlockPos.getAllInBox(new BlockPos(pos.getX() - 1, pos.getY() - 1, pos.getZ() - 1), new BlockPos(pos.getX() + 1, pos.getY() + 1, pos.getZ() + 1))) {
        			boolean isLog = isValidLogBlock(world, newPos, startBlock);
        			if ((isLog) && (!positions.contains(newPos)) && (!res.contains(newPos))) {
        				positions.push(newPos);
        			}
        		}
        	}
        } 
        while (!positions.empty());
        return res;
	}
  
	private boolean isValidLogBlock(World world, BlockPos pos, Block start) {
		IBlockState state = world.getBlockState(pos);
		Block block = state.getBlock();
		if (start != null) {
			return block == start;
		}
		return (block != null) && (block instanceof BlockLog || block instanceof BlockNewLog);
	}
     
	private List<BlockPos> getConnectedOreBlocks(World world, BlockPos blockPos) {
		List<BlockPos> res = new ArrayList();
		Stack<BlockPos> positions = new Stack();
		positions.push(blockPos);
        IBlockState start = world.getBlockState(blockPos);
        Block startBlock = start.getBlock();
        do {
        	BlockPos pos = (BlockPos)positions.pop();
        	res.add(pos);
        	if (pos.distanceSq(blockPos) < 256.0D) { // find all connected ore within 16 blocks
        		for (BlockPos newPos : BlockPos.getAllInBox(new BlockPos(pos.getX() - 1, pos.getY() - 1, pos.getZ() - 1), new BlockPos(pos.getX() + 1, pos.getY() + 1, pos.getZ() + 1))) {
        			boolean isOre = isValidOreBlock(world, newPos, startBlock);
        			if ((isOre) && (!positions.contains(newPos)) && (!res.contains(newPos))) {
        				positions.push(newPos);
        			}
        		}
        	}
        } 
        while (!positions.empty());
        return res;
	}
  
	private boolean isValidOreBlock(World world, BlockPos pos, Block start) {
		IBlockState state = world.getBlockState(pos);
		Block block = state.getBlock();
		if (start != null) {
			return block == start;
		}
		return (block != null) && (block instanceof BlockOre || block instanceof BlockRedstoneOre);
	}
  
   	@Override
	public boolean getIsRepairable(ItemStack toRepair, ItemStack repair) {
		if (toRepair.getItem().equals(ItemsCreate.mithril_paxel)) {
			if (repair.getItem().equals(ItemsCreate.mithril_ingot)) {
				return true;
			}
		}
		return false;
	}

}
