package rainbeau.mithwoodforest.RMFItems;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Stack;

import com.google.common.collect.Sets;

import net.minecraft.block.Block;
import net.minecraft.block.BlockOre;
import net.minecraft.block.BlockRedstoneOre;
import net.minecraft.block.material.Material;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Blocks;
import net.minecraft.item.Item;
import net.minecraft.item.ItemPickaxe;
import net.minecraft.item.ItemStack;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;
import net.minecraftforge.fml.common.registry.GameRegistry;
import rainbeau.mithwoodforest.RMFBlocks.BlocksCreate;

public class ItemPickaxeMithril extends ItemPickaxe {
	
    private static final Set<Block> EFFECTIVE_ON = Sets.newHashSet(new Block[] {Blocks.ACTIVATOR_RAIL, Blocks.COAL_ORE, Blocks.COBBLESTONE, Blocks.DETECTOR_RAIL, Blocks.DIAMOND_BLOCK, Blocks.DIAMOND_ORE, Blocks.DOUBLE_STONE_SLAB, Blocks.GOLDEN_RAIL, Blocks.GOLD_BLOCK, Blocks.GOLD_ORE, Blocks.ICE, Blocks.IRON_BLOCK, Blocks.IRON_ORE, Blocks.LAPIS_BLOCK, Blocks.LAPIS_ORE, Blocks.LIT_REDSTONE_ORE, Blocks.MOSSY_COBBLESTONE, Blocks.NETHERRACK, Blocks.PACKED_ICE, Blocks.RAIL, Blocks.REDSTONE_ORE, Blocks.SANDSTONE, Blocks.RED_SANDSTONE, Blocks.STONE, Blocks.STONE_SLAB, Blocks.STONE_BUTTON, Blocks.STONE_PRESSURE_PLATE});

	public ItemPickaxeMithril(String unlocalizedName, Item.ToolMaterial material) {
	    super(material);
	    this.setUnlocalizedName(unlocalizedName);
	    this.setRegistryName(unlocalizedName);
	    GameRegistry.register(this);
	}
	
	@Override
	public boolean onBlockStartBreak(ItemStack stack, BlockPos blockPos, EntityPlayer player) {
		World world = player.worldObj;
		IBlockState firstblock = world.getBlockState(blockPos);
		if (firstblock.getBlock() instanceof BlockOre || firstblock.getBlock() instanceof BlockRedstoneOre) {
			List<BlockPos> oreBlocks = getConnectedOreBlocks(world, blockPos);
			for (BlockPos pos : oreBlocks) {
				if (!pos.equals(blockPos)) {
					IBlockState block = world.getBlockState(pos);
					if (block.getBlock() instanceof BlockOre || block.getBlock() instanceof BlockRedstoneOre) {
						world.setBlockToAir(pos);
						block.getBlock().dropBlockAsItem(world, pos, block, 0);
					}
				}
			}
		}
		stack.damageItem(1, player);
		return false;
	}
  
	@Override
	public boolean onBlockDestroyed(ItemStack stack, World worldIn, IBlockState state, BlockPos pos, EntityLivingBase entityLiving) {
		return true;
	}
  
	private List<BlockPos> getConnectedOreBlocks(World world, BlockPos blockPos) {
		List<BlockPos> res = new ArrayList();
		Stack<BlockPos> positions = new Stack();
		positions.push(blockPos);
        IBlockState start = world.getBlockState(blockPos);
        Block startBlock = start.getBlock();
        do {
        	BlockPos pos = (BlockPos)positions.pop();
        	res.add(pos);
        	if (pos.distanceSq(blockPos) < 256.0D) { // find all connected ore within 16 blocks
        		for (BlockPos newPos : BlockPos.getAllInBox(new BlockPos(pos.getX() - 1, pos.getY() - 1, pos.getZ() - 1), new BlockPos(pos.getX() + 1, pos.getY() + 1, pos.getZ() + 1))) {
        			boolean isOre = isValidOreBlock(world, newPos, startBlock);
        			if ((isOre) && (!positions.contains(newPos)) && (!res.contains(newPos))) {
        				positions.push(newPos);
        			}
        		}
        	}
        } 
        while (!positions.empty());
        return res;
	}
  
	private boolean isValidOreBlock(World world, BlockPos pos, Block start) {
		IBlockState state = world.getBlockState(pos);
		Block block = state.getBlock();
		if (start != null) {
			return block == start;
		}
		return (block != null) && (block instanceof BlockOre || block instanceof BlockRedstoneOre);
	}
 
	@Override
	public boolean canHarvestBlock(IBlockState blockIn) {
		Block block = blockIn.getBlock();
		if (block == BlocksCreate.unobtainium_ore) {
			return this.toolMaterial.getHarvestLevel() >= 4;
		}
		else if (block == Blocks.OBSIDIAN) {
			return this.toolMaterial.getHarvestLevel() >= 3;
		}
		else if (block != Blocks.DIAMOND_BLOCK && block != Blocks.DIAMOND_ORE) {
			if (block != Blocks.EMERALD_ORE && block != Blocks.EMERALD_BLOCK) {
				if (block != Blocks.GOLD_BLOCK && block != Blocks.GOLD_ORE) {
					if (block != Blocks.IRON_BLOCK && block != Blocks.IRON_ORE) {
						if (block != Blocks.LAPIS_BLOCK && block != Blocks.LAPIS_ORE) {
							if (block != Blocks.REDSTONE_ORE && block != Blocks.LIT_REDSTONE_ORE) {
								Material material = blockIn.getMaterial();
								return material == Material.ROCK ? true : (material == Material.IRON ? true : material == Material.ANVIL);
							}
							else {
								return this.toolMaterial.getHarvestLevel() >= 2;
							}
						}
						else {
							return this.toolMaterial.getHarvestLevel() >= 1;
						}
					}
					else {
						return this.toolMaterial.getHarvestLevel() >= 1;
					}
				}
				else {
					return this.toolMaterial.getHarvestLevel() >= 2;
				}
			}
			else {
				return this.toolMaterial.getHarvestLevel() >= 2;
			}
		}
		else {
			return this.toolMaterial.getHarvestLevel() >= 2;
		}
	}

	@Override
	public boolean getIsRepairable(ItemStack toRepair, ItemStack repair) {
		if (toRepair.getItem().equals(ItemsCreate.mithril_pickaxe)) {
			if (repair.getItem().equals(ItemsCreate.mithril_ingot)) {
				return true;
			}
		}
		return false;
	}

}
