package rainbeau.mithwoodforest.RMFWorldGen;

import java.util.Random;

import net.minecraft.block.BlockDoublePlant;
import net.minecraft.block.BlockFlower;
import net.minecraft.block.BlockTallGrass;
import net.minecraft.entity.passive.EntityOcelot;
import net.minecraft.entity.passive.EntityRabbit;
import net.minecraft.entity.passive.EntityWolf;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.MathHelper;
import net.minecraft.world.World;
import net.minecraft.world.biome.Biome;
import net.minecraft.world.gen.feature.WorldGenAbstractTree;
import net.minecraft.world.gen.feature.WorldGenBigTree;
import net.minecraft.world.gen.feature.WorldGenBirchTree;
import net.minecraft.world.gen.feature.WorldGenMelon;
import net.minecraft.world.gen.feature.WorldGenTaiga2;
import net.minecraft.world.gen.feature.WorldGenTallGrass;
import net.minecraft.world.gen.feature.WorldGenTrees;
import net.minecraft.world.gen.feature.WorldGenerator;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

public class BiomeMithwoodForest extends Biome {
	
    public BiomeMithwoodForest() {
 		super(new BiomeProperties("Mithwood Forest").setTemperature(0.7F).setRainfall(0.8F).setWaterColor(0x70ff70));
 		this.theBiomeDecorator.bigMushroomsPerChunk = 1;
 		this.theBiomeDecorator.clayPerChunk = 1;
        this.theBiomeDecorator.deadBushPerChunk = 2;
        this.theBiomeDecorator.flowersPerChunk = 16;
        this.theBiomeDecorator.grassPerChunk = 4;
        this.theBiomeDecorator.mushroomsPerChunk = 8;
        this.theBiomeDecorator.reedsPerChunk = 4;
        this.theBiomeDecorator.sandPerChunk = 1;
        this.theBiomeDecorator.sandPerChunk2 = 1;
        this.theBiomeDecorator.treesPerChunk = 8;
        this.theBiomeDecorator.waterlilyPerChunk = 4;
        this.spawnableCreatureList.clear();
        this.spawnableCreatureList.add(new Biome.SpawnListEntry(EntityOcelot.class, 2, 1, 1));
        this.spawnableCreatureList.add(new Biome.SpawnListEntry(EntityRabbit.class, 8, 2, 3));
        this.spawnableCreatureList.add(new Biome.SpawnListEntry(EntityWolf.class, 4, 4, 4));
    }

    @Override
    public WorldGenerator getRandomWorldGenForGrass(Random rand) {
        return rand.nextInt(4) == 0 ? new WorldGenTallGrass(BlockTallGrass.EnumType.FERN) : new WorldGenTallGrass(BlockTallGrass.EnumType.GRASS);
    }

	@Override
	public WorldGenAbstractTree genBigTreeChance(Random rand) {
        int treeType = rand.nextInt(20);
        if (treeType == 0) {
        	return new WorldGenMithwoodTreeBig(false);
        }
        else if (treeType < 3) {
        	return new WorldGenMithwoodTreeSmall(false);
        }
        else if (treeType < 5) {
        	int treeTypeA = rand.nextInt(4);
        	if (treeTypeA == 0) {
        		return new WorldGenBirchTree(false,true);
        	}
        	else {
        		return new WorldGenBigTree(false);
        	}
        }
        else if (treeType < 9) {
        	return new WorldGenTaiga2(false);
        }
        else if (treeType < 13) {
        	return new WorldGenBirchTree(false, false);
        }
        else {
        	return new WorldGenTrees(false);
        }
	}
		
    @Override
    public BlockFlower.EnumFlowerType pickRandomFlower(Random rand, BlockPos pos) {
        double d0 = MathHelper.clamp_double((1.0D + GRASS_COLOR_NOISE.getValue((double)pos.getX() / 48.0D, (double)pos.getZ() / 48.0D)) / 2.0D, 0.0D, 0.9999D);
        BlockFlower.EnumFlowerType blockflower$enumflowertype = BlockFlower.EnumFlowerType.values()[(int)(d0 * (double)BlockFlower.EnumFlowerType.values().length)];
        return blockflower$enumflowertype == BlockFlower.EnumFlowerType.BLUE_ORCHID ? BlockFlower.EnumFlowerType.POPPY : blockflower$enumflowertype;
    }

    @Override
    public void decorate(World worldIn, Random rand, BlockPos pos) {
        int p = rand.nextInt(5) - 1;
        this.addDoublePlants(worldIn, rand, pos, p);
        super.decorate(worldIn, rand, pos);
        int i = rand.nextInt(16) + 8;
        int j = rand.nextInt(16) + 8;
        int height = worldIn.getHeight(pos.add(i, 0, j)).getY() * 2; 
        if (height < 1) height = 1;
        int k = rand.nextInt(height);
        (new WorldGenMelon()).generate(worldIn, rand, pos.add(i, k, j));
    }
    
    public void addDoublePlants(World p_185378_1_, Random p_185378_2_, BlockPos p_185378_3_, int p_185378_4_) {
        for (int i = 0; i < p_185378_4_; ++i) {
            int j = p_185378_2_.nextInt(3);
            if (j == 0) {
                DOUBLE_PLANT_GENERATOR.setPlantType(BlockDoublePlant.EnumPlantType.SYRINGA);
            }
            else if (j == 1) {
                DOUBLE_PLANT_GENERATOR.setPlantType(BlockDoublePlant.EnumPlantType.ROSE);
            }
            else if (j == 2) {
                DOUBLE_PLANT_GENERATOR.setPlantType(BlockDoublePlant.EnumPlantType.PAEONIA);
            }
            for (int k = 0; k < 5; ++k) {
                int l = p_185378_2_.nextInt(16) + 8;
                int i1 = p_185378_2_.nextInt(16) + 8;
                int j1 = p_185378_2_.nextInt(p_185378_1_.getHeight(p_185378_3_.add(l, 0, i1)).getY() + 32);
                if (DOUBLE_PLANT_GENERATOR.generate(p_185378_1_, p_185378_2_, new BlockPos(p_185378_3_.getX() + l, j1, p_185378_3_.getZ() + i1))) {
                    break;
                }
            }
        }
    }

    @Override
    public void addDefaultFlowers() {
    	for (BlockFlower.EnumFlowerType type : BlockFlower.EnumFlowerType.values()) {
    		if (type.getBlockType() == BlockFlower.EnumFlowerColor.YELLOW) continue;
    		if (type == BlockFlower.EnumFlowerType.BLUE_ORCHID) type = BlockFlower.EnumFlowerType.POPPY;
    		addFlower(net.minecraft.init.Blocks.RED_FLOWER.getDefaultState().withProperty(net.minecraft.init.Blocks.RED_FLOWER.getTypeProperty(), type), 10);
    	}
    }

    @Override
    public Class <? extends Biome > getBiomeClass() {
        return BiomeMithwoodForest.class;
    }

    @Override
    @SideOnly(Side.CLIENT)
    public int getGrassColorAtPos(BlockPos pos) {
   		return 0x407080;
    }
    
}
